import re

from text2art.utils import colorFormat


def colored(text, color=None, on_color=None, attr=None):
    """Dyeing the input text.
    
    
    The original format is removed from the input text, 
    and the new specified format is applied.
    
    Args:
        text: Get an character string.
        color: Get a color string,dye the input string 
               with the corresponding color.
        available text colors:
               red, green, yellow, blue, magenta, cyan, white.
        on_color: Get an character string,setting the background 
                  color of the text.
        available text highlights:
            on_red, on_green, on_yellow, on_blue, on_magenta, on_cyan, 
            on_white.
        attr: Get a character string,setting the effect of the text.
        available attributes:
                bold, dark, underline, blink, reverse, concealed.
                
    Returns:
        A text of a specific color effect.
        
    Example:
        colored("Hello, Yang!", "red", "on_grey", "bold")
        colored("Hello, Yang!", "green")
    """

    
    if color is not None:
        # replace the original color font,example: "\x1b[36mHang\x1b[0m" -> "Hang".
        text = re.sub(colorFormat.COLORS_REGEX + "(.*?)" + colorFormat.RESET_REGEX, r"\1", text)
        text = colorFormat.FMT_STR % (colorFormat.COLORS[color], text)
        
    if on_color is not None:
        # replace a font with a background color.
        text = re.sub(colorFormat.HIGHLIGHTS_REGEX + "(.*?)" + colorFormat.RESET_REGEX, r"\1", text)
        text = colorFormat.FMT_STR % (colorFormat.HIGHLIGHTS[on_color], text)
        
    if attr is not None:
        # replace the effect of the original font.
        text = re.sub(colorFormat.ATTRIBUTES_REGEX + "(.*?)" + colorFormat.RESET_REGEX, r"\1", text)
        text = colorFormat.FMT_STR % (colorFormat.ATTRIBUTES[attr], text)
            
    return text + colorFormat.RESET


import random
from pyfiglet import Figlet, FigletFont

import fire
from colorama import init

from text2art.color import colored
from text2art.utils import colorFormat

DEFAULT_FONT = "ghost"
font_list = FigletFont.getFonts()
# Key value pair reversal
color_dict = {value: key for key, value in colorFormat.COLORS.items()}

def lf():
    """Random display of 25 fonts"""
    return random.sample(font_list, 25)
    
def rd(text, on_color=None, attr=None, 
       width=80, justify="center"):
    """An art font that generates random fonts 
       and random colors.
       
       
       Args:
        text: Get an character string.
        color: Get a color string,dye the input string 
               with the corresponding color.
        available text colors:
               red, green, yellow, blue, magenta, cyan, white.
        on_color: Get an character string,setting the background 
                  color of the text.
        available text highlights:
            on_red, on_green, on_yellow, on_blue, on_magenta, on_cyan, 
            on_white.
        attr: Get a character string,setting the effect of the text.
        available attributes:
                bold, dark, underline, blink, reverse, concealed.
        width: Setting the terminal width of the output font, type is int.
        justify: Setting the location of the terminal output font.
        available parameter: left, enter, right.
    Returns:
        A text of a specific color effect.
    
    """
    rand_int = random.randint(1, len(font_list)+1)
    rand_color = color_dict.get(random.randint(30, 38))
    
    rand_font = font_list[rand_int]
    print("Random font is :{}".format(rand_font))
    f = Figlet(
            font=rand_font, width=width, 
            justify=justify
        )
    r = f.renderText(text)
    return colored(r, rand_color, on_color, attr)

def gt(text, font=DEFAULT_FONT, color="magenta", 
       on_color=None, attr=None, width=80, 
       justify="center"):
       
    """An art font that generates the effect of 
       the specified parameter.
       
       
       Args:
        text: Get an character string.
        on_color: Get an character string,setting the background 
                  color of the text.
        available text highlights:
            on_red, on_green, on_yellow, on_blue, on_magenta, on_cyan, 
            on_white.
        attr: Get a character string,setting the effect of the text.
        available attributes:
                bold, dark, underline, blink, reverse, concealed.
        width: Setting the size of the terminal output font, type is int.
        justify: Setting the location of the terminal output font.
        available parameter: left, enter, right.
    Returns:
        A text of a specific color effect.
    
    """
       
    f = Figlet(
            font, width=width,
            justify=justify
        )
    r = f.renderText(text)
    return colored(r, color, on_color, attr)    

def h():
    doc = """
            Usage:
                text2art lf  # Random display of 25 fonts
                text2art rd text [--on_color] [--attr] [--width] [--justify]
                text2art gt text [--font] [--color] [--on_color] [--attr] [--width] [--justify]
            
            available text colors:
                red, green, yellow, blue, magenta, cyan, white.
            available text highlights:
                on_red, on_green, on_yellow, on_blue, on_magenta, 
                on_cyan,on_white.
            available attributes:
                bold, dark, underline, blink, reverse, concealed.
            width: Setting the size of the terminal output font,type is int.
            justify: Setting the location of the terminal output font.
            available parameter: left, enter, right.
          """
    print(doc)
    
def main():
    init(autoreset=True)
    fire.Fire()

if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

VERSION = "0.1.9"

with open("README.md", "r", encoding="utf-8") as fp:
    long_description = fp.read()

setup(
    name = "text2art",
    version = VERSION,
    description = "Generating controlled terminal art fonts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license = "MIT",
    include_package_data = True,
    author = "hangfengYang",
    author_email = "yhf5fhy@gmail.com",
    classifiers = [],
    url = "https://github.com/Fenghuapiao/text2art",
    packages = ["text2art"],
    install_requires = [
        "fire",
        "colorama",
        "pyfiglet"
    ],
    entry_points = {
        "console_scripts": [
            "text2art  =  text2art.main:main",
        ],
    }
)




class colorFormat:
    """Display color format

        `\033[display mode; font color; background color m String\033[0m`

        -------------------------------------------------
        Font color| background color | color description
        -------------------------------------------------
        30        |        40        |       grey
        31        |        41        |       red
        32        |        42        |       green
        33        |        43        |       yellow
        34        |        44        |       blue
        35        |        45        |       magenta
        36        |        46        |       cyan
        37        |        47        |       white
        -------------------------------------------------
        ------------------------------------------------
        display mode   |     effect
        ------------------------------------------------
        0              |     terminal default settings
        1              |     highlight
        4              |     use a underline
        5              |     twinkle
        7              |     anti white display
        8              |     invisible
        ------------------------------------------------
    """
   
    COLORS = dict(
                list(
                    zip([
                        "grey",
                        "red",
                        "green",
                        "yellow",
                        "blue",
                        "magenta",
                        "cyan",
                        "white"
                        ],
                        list(range(30, 38))
                    )
                )
        )
        
    HIGHLIGHTS = dict(
                    list(
                        zip([
                            "on_grey",
                            "on_red",
                            "on_green",
                            "on_yellow",
                            "on_blue",
                            "on_magenta",
                            "on_cyan",
                            "on_white"
                            ],
                            list(range(40, 48))
                        )
                    )
        )
        
    ATTRIBUTES = dict(
                    list(
                        zip([
                            "bold",
                            "dark",
                            "",
                            "underline",
                            "blink",
                            "",
                            "reverse",
                            "concealed"
                            ],
                            list(range(1, 9))
                        )
                    )
        )
      
    del ATTRIBUTES[""]
    
    RESET = "\033[0m"
    FMT_STR = "\033[%dm%s"
    RESET_REGEX = "\033\[0m"
    COLORS_REGEX = "\033\[(?:{:s})m".format("|".join(["%d" % v for v in COLORS.values()]))
    HIGHLIGHTS_REGEX = "\033\[(?:{:s})m".format("|".join(["%d" % v for v in HIGHLIGHTS.values()]))
    ATTRIBUTES_REGEX = "\033\[(?:{:s})m".format("|".join(["%d" % v for v in ATTRIBUTES.values()]))
